//jTimer adapted from f. losasso

#ifndef jTimer_h
#define jTimer_h

#include <stdio.h>
#include <windows.h>

#pragma comment(lib, "winmm.lib")

#define MAX_TIMERS	8

class timer {
	private:
		__int64	frequency;
		float resolution;
		bool performanceTimer;
		unsigned long	mmTimerStart    [MAX_TIMERS];	
		unsigned long	mmTimerElapsed  [MAX_TIMERS];	
		__int64	perfTimerStart  [MAX_TIMERS];	
		__int64	perfTimerElapsed[MAX_TIMERS];	
		int numTimers;
		static timer*	singletonInstance;
		timer();
	public:
	static inline timer* singleton() {
		if (!singletonInstance)
			singletonInstance=new timer();
		return singletonInstance;
	}
	~timer() {};
	double getTime();
	void registerTimer(int* id);
	float getTotalTime(int id);
	float resetTime (int id);
	float peekTime (int id);
};

timer* timer::singletonInstance=NULL;

timer::timer() {
	numTimers=1;
	perfTimerElapsed[0]=0;
	perfTimerStart[0]=0;
	if (!QueryPerformanceFrequency((LARGE_INTEGER *) &frequency))	{		
		performanceTimer=false;
		resolution=1.0f/1000.0f;
		frequency=1000;
	} else {		
		performanceTimer=true;		
		resolution=(float)(((double)1.0f)/((double)frequency));
	}
};

void timer::registerTimer(int* ID) {
	if (numTimers >= MAX_TIMERS) {
		printf("No more timers available\n");
		fflush(stdout);
		exit(0);
	}	
	*ID=numTimers++;
	if (performanceTimer) {
		__int64 time;
		QueryPerformanceCounter((LARGE_INTEGER *) &time);
		perfTimerStart[*ID] = time;
		perfTimerElapsed[*ID] = time;
	} else {		
		mmTimerStart[*ID] = timeGetTime();		
		mmTimerElapsed[*ID] = timeGetTime();
	}
	return;
};

double timer::getTime()
{
	__int64 time;	
	if (performanceTimer) {		
		QueryPerformanceCounter((LARGE_INTEGER *) &time);		
		return ( (double) time * resolution)*1000.0;
	} else
		return ( (double) timeGetTime() * resolution)*1000.0;
};

float timer::getTotalTime(int ID) {
	__int64 time;
	if (performanceTimer) {		
		QueryPerformanceCounter((LARGE_INTEGER *) &time);		
		return ( (float) ( time - perfTimerStart[ID]) * resolution)*1000.0f;
	} else	
		return ( (float) ( timeGetTime() - mmTimerStart[ID]) * resolution)*1000.0f;
};

float timer::resetTime(int ID) {
	__int64 time;
	float timeElapsed;
	if (performanceTimer) {		
		QueryPerformanceCounter((LARGE_INTEGER*)&time);
		timeElapsed=((float)(time-perfTimerElapsed[ID])*resolution)*1000.0f;		
		perfTimerElapsed[ID]=time;		
		return timeElapsed;
	}	else {		
		timeElapsed = ((float)(timeGetTime()-mmTimerElapsed[ID])*resolution)*1000.0f;
		mmTimerElapsed[ID] = timeGetTime();		
		return timeElapsed;
	}
};

float timer::peekTime(int ID) {
	__int64 time;
	float timeElapsed;
	if (performanceTimer) {		
		QueryPerformanceCounter((LARGE_INTEGER *) &time);		
		timeElapsed = ((float)(time - perfTimerElapsed[ID])*resolution)*1000.0f;
	} else
		timeElapsed = ((float)(timeGetTime() - mmTimerElapsed[ID])*resolution)*1000.0f;	
	return timeElapsed;
};

#endif
